package org.zeroturnaround.javarebel.integration.spring;

import org.zeroturnaround.javarebel.StopWatch;

import java.io.File;
import org.springframework.beans.factory.support.BeanDefinitionReader;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.core.io.Resource;
import org.zeroturnaround.javarebel.LoggerFactory;

class XMLBeanDefinitionResource {
  private long lastModified;
  private Resource xmlRes;
  private File xmlFile;
  private BeanDefinitionReader bdr;

  public XMLBeanDefinitionResource(BeanDefinitionReader bdr, Resource xmlRes, File xmlFile) { 
    this.bdr = bdr;
    this.xmlFile = xmlFile;
    this.xmlRes = xmlRes;

    lastModified = xmlFile.lastModified();   
  }

  public synchronized void reload() {
    StopWatch sw = 
      LoggerFactory.getInstance().createStopWatch("XMLBeanDefinitionResource.reload");

    try {
      LoggerFactory.getInstance().echo(
          "JavaRebel-Spring: Reloading Spring bean definitions in '" + xmlFile + "'.");

      try {
        bdr.loadBeanDefinitions(xmlRes);
        
        if (bdr.getRegistry() instanceof DefaultListableBeanFactory) {
          // XXX Is this safe? See {@link DefaultListableBeanFactoryCBP}
          ((DefaultListableBeanFactory) bdr.getRegistry()).preInstantiateSingletons();
        }
      }
      catch (Throwable e) {
        if (e instanceof Error) throw (Error) e;
        LoggerFactory.getInstance().error(e);
      }
    }
    finally {
      if (sw != null) sw.stop();
    }
  }


  public synchronized void checkAndReload() {
    if (xmlFile.lastModified() > lastModified) {
      lastModified = xmlFile.lastModified();
      reload();
    }
  }
}