package org.zeroturnaround.javarebel.integration.spring.cbp;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Registers the SpringBeanReloader for each bean factory 
 * and call checkAndReload() on all classes being configured.
 */
public class AbstractBeanFactoryCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.springframework.beans.factory.config");
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.integration.spring");    
    
    CtConstructor[] ctrs = ctClass.getConstructors();
    for (int i = 0; i < ctrs.length; i++) {
      ctrs[i].insertAfter(
          "      if ($0 instanceof ConfigurableListableBeanFactory) {\n" + 
          "        ReloaderFactory.getInstance().addClassReloadListener(" +
          "            new SpringBeanReloader((ConfigurableListableBeanFactory) $0));\n" + 
          "      }");      
    }

    CtMethod m = ctClass.getDeclaredMethod("getObjectForBeanInstance");   
    m.insertBefore("if ($1 != null) {ReloaderFactory.getInstance().checkAndReload($1.getClass());}");
    
    
  }
}
