package org.zeroturnaround.javarebel.integration.spring.cbp;

import org.zeroturnaround.javarebel.ClassEventListener;

import org.zeroturnaround.bundled.javassist.CtNewMethod;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Disables Spring introspection cache for JavaBeans Introspector results used in
 * resolving XML dependencies.
 */
public class CachedIntrospectionResultsCBP extends JavassistClassBytecodeProcessor {

	public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
	  cp.importPackage("java.beans");
	  cp.importPackage("java.util");
	  cp.importPackage("org.springframework.beans");
	  cp.importPackage("org.springframework.core");  
	  cp.importPackage("org.zeroturnaround.javarebel");
	  
	  // Empties cache
	  ctClass.addInterface(cp.get(ClassEventListener.class.getName()));	  
    CtConstructor init = ctClass.getDeclaredConstructor(cp.get(new String[] {Class.class.getName()}));
    init.insertAfter(
        "ReloaderFactory.getInstance().addClassReloadListener($0);");   
    
    CtMethod priorityMethod = CtNewMethod.make("public int priority() {\n" + 
    		"    return PRIORITY_CORE + 9999;\n" + 
    		"  }", ctClass);
    
    ctClass.addMethod(priorityMethod);
    
    CtMethod onClassEventMethod = CtNewMethod.make(
        " public void onClassEvent(int eventType, Class beanClass) {\n" + 
        "       if (beanClass != getBeanClass()) return; " +
        "       " +
        "       try {\n" + 
        "      this.beanInfo = Introspector.getBeanInfo(beanClass);\n" + 
        "\n" + 
        "      Class classToFlush = beanClass;\n" + 
        "      do {\n" + 
        "        Introspector.flushFromCaches(classToFlush);\n" + 
        "        classToFlush = classToFlush.getSuperclass();\n" + 
        "      }\n" + 
        "      while (classToFlush != null);\n" + 
        "\n" + 
        "      this.propertyDescriptorCache = new HashMap();\n" + 
        "\n" + 
        "      // This call is slow so we do it once.\n" + 
        "      PropertyDescriptor[] pds = $0.beanInfo.getPropertyDescriptors();\n" + 
        "      for (int i = 0; i < pds.length; i++) {\n" + 
        "        PropertyDescriptor pd = pds[i];\n" +
        "         " +                 
//        "        if (JdkVersion.isAtLeastJava15()) {\n" + 
//        "          pd = new GenericTypeAwarePropertyDescriptor(beanClass, pd.getName(),\n" + 
//        "              pd.getReadMethod(), pd.getWriteMethod(), pd.getPropertyEditorClass());\n" + 
//        "        }\n" + 
        "        this.propertyDescriptorCache.put(pd.getName(), pd);\n" + 
        "      }\n" + 
        "    }\n" + 
        "    catch (IntrospectionException ex) {\n" + 
        "      throw new FatalBeanException(\"Cannot get BeanInfo for object of class [\" + beanClass.getName() + \"]\", ex);\n" + 
        "    }\n" + 
        "" + 
        "  }",
        ctClass);
    ctClass.addMethod(onClassEventMethod);    
	}
}
