package org.zeroturnaround.javarebel.integration.spring.cbp;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.ClassEventListener;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Disables Spring annotation cache for dependency method and field annotations.
 */
public class InjectionMetadataCacheCBP extends JavassistClassBytecodeProcessor {

	public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {	   
	 cp.importPackage("org.zeroturnaround.javarebel");
	  
	// Empties cache
    ctClass.addInterface(cp.get(ClassEventListener.class.getName()));   
    CtConstructor init = ctClass.getDeclaredConstructor(new CtClass[] {});
    init.insertAfter(
        "ReloaderFactory.getInstance().addClassReloadListener($0);");   
    
    CtMethod priorityMethod = CtNewMethod.make("public int priority() {\n" + 
        "    return PRIORITY_CORE + 9900;\n" + 
        "  }", ctClass);
    
    ctClass.addMethod(priorityMethod);
    
    // Spring 2.0.8 has "annotationMetadataCache" instead in PersistenceAnnotationBeanPostProcessor.
    boolean hasInjectionMetadataCache = true;
    try {
      ctClass.getDeclaredField("injectionMetadataCache");      
    } catch (NotFoundException e) {
      hasInjectionMetadataCache = false;
    }
    
    CtMethod onClassEventMethod = CtNewMethod.make(
        " public void onClassEvent(int eventType, Class beanClass) {\n" +
        "   " + (hasInjectionMetadataCache ? "injectionMetadataCache" : "annotationMetadataCache") + ".remove(beanClass);" + 
        " }",
        ctClass);
    ctClass.addMethod(onClassEventMethod);    
	}
}
